// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: Array
// Author: Hassan Siddiqui
//
// Description: 
//
// Class representation of a PixelIterator that handles arrays
//
// Modifications: See ChangeLog
// 
// $Id: ArrayPixelIterator.h,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

#ifndef ARRAYPIXELITERATOR_H
#define ARRAYPIXELITERATOR_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"
#include "ConvertableData.h"
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "PixelIterator.h"
#include "WCSConvertor.h"
#include "PixelElement.h"

class ArrayPixelIterator : public PixelIterator
{
public:

  /* Constructors */
  ArrayPixelIterator(const string & arrayname);

  /* Destructors */
  ~ArrayPixelIterator();

  /* Iterator methods */
  void first();
  void next ();
  void setIndex ( unsigned long index);     // Sets the array position
  bool isDone();
  PixelElement currentItem();

  /* WCSConvertor object */
  const WCSConvertor * wcsconvertor() const;
  
  /* The total number of elements in map */
  unsigned size() const;

  /* The number of bins in x and y-direction 
     (not a generic PixelIterator method) */
  unsigned xbins() const;
  unsigned ybins() const;

  /* Get the value of a given array pixel without setting the current item */
  double valueAt(unsigned long index);
  double valueAt(); // value at the current pixel

  // Set these members protected so that derived classes can access them
protected:
  bool _firstcalled;
  unsigned long _index;
  unsigned long _xindex;
  unsigned long _yindex;
  unsigned long _xbins;
  unsigned long _size;

  DataSet *      _set;
  Array *        _array;
  ArrayData *    _arraydata;
  ConvertableData _arrayvalue;
  WCSConvertor * _wcsconv;

private:

  /* Disallow default constructor */
  ArrayPixelIterator();

};

#endif
