// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*

    ConvertableData

    Class that allows access to elements of a DAL Data object 
    regardless of its type.

    (c) 1998 European Space Agency - written by Marco Beijersbergen
    
    Moved to separate utils package by H Siddiqui

*/

/* SAS */
#include <Dal.h>
#include <errstr.h>

/* C++ */
#include <fstream> 
#include <set>

/* C */
#include <cstdlib>


class ConvertableData
{

  public:

    ConvertableData() : _data(0) {}

    ConvertableData(Data* d) { data(d); }

    void data(Data* d);

    double asDouble(unsigned row);

    string operator[](unsigned row);

  private:

    Dal::DataType _type;

    void *_data;
};


// Utility function to determine whether a column is numeric or not.

static Dal::DataType numericTypesArray[] =
	{ Dal::Int8, Dal::Int16, Dal::Int32, Dal::Real32, Dal::Real64 };
static set<Dal::DataType,less<Dal::DataType> > numericTypes(
	&numericTypesArray[0],
	&numericTypesArray[sizeof(numericTypesArray)
		/sizeof(numericTypesArray[0])]);

bool isNumeric(Dal::DataType tp);
