// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: NullPixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Class representation of an iterator that handles DAL-based detectormaps.
//
// Modifications: See ChangeLog
// 
// $Id: NullPixelIterator.cc,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

// General classes
#include <iostream>
#include<cassert>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "NullPixelIterator.h"

/* Constructors */
NullPixelIterator::NullPixelIterator() :
  _firstcalled(false),
  _isdone(false),
  _wcsconv(0)
{ 
  /* Create WcsConvertor object */
  _wcsconv = new WCSConvertor();

  normalise();
}

/* Destructor */
NullPixelIterator::~NullPixelIterator()
{
  PROC;
  delete _wcsconv;   _wcsconv=0;
  ENDPROC;
}


/* Iterator methods */
void NullPixelIterator::first()
{
  _firstcalled = true;
}


void NullPixelIterator::next()
{
  assert (!isDone());
  _isdone = true;
}

bool NullPixelIterator::isDone()
{
  PROC;
  assert(_firstcalled);
  RETURN (_isdone);
}


PixelElement NullPixelIterator::currentItem()
{
  PROC;
  assert(_firstcalled);
  RETURN PixelElement(1,1,1.,1.,1., 1., 1.);
}


unsigned NullPixelIterator::size() const
{
  PROC;
  RETURN 1;
}

unsigned NullPixelIterator::xbins() const
{
  PROC;
  RETURN 1;
}

/* WCSConvertor object */
const WCSConvertor * NullPixelIterator::wcsconvertor() const
{
  PROC;
  RETURN _wcsconv;
}

