// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: NullPixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Class that does not iterate through a pixel structure as such; it only
// iterates once and returns a single PixelElement object.
// Used in cases where an iterator is required, 
// but no real Detmap is specified.
//
// Modifications: See ChangeLog
// 
// $Id: NullPixelIterator.h,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

#ifndef NULLPIXELITERATOR_H
#define NULLPIXELITERATOR_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "PixelIterator.h"
#include "WCSConvertor.h"
#include "PixelElement.h"

class NullPixelIterator : public PixelIterator
{
public:

  /* Constructors */
  NullPixelIterator();

  /* Destructors */
  ~NullPixelIterator();

  /* Iterator methods */
  void   first();
  void    next ();
  bool   isDone();
  PixelElement currentItem();

  /* Size methods */
  unsigned    size()  const;
  unsigned    xbins() const;

  /* WCSConvertor object */
  const WCSConvertor * wcsconvertor() const;

private:
  bool _firstcalled;
  bool _isdone;
  WCSConvertor * _wcsconv;

};

#endif
