// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Allows one to iterate across a structure of pixels.
// Useful for traversing detector maps
// as described in email XMM-SOC/HS/1999-100,
// or detector masks.
//
// Modifications: See ChangeLog
// 
// $Id: PixelIterator.cc,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

/* SAS */
#include "Tracer.h"

/* Local classes */
#include "PixelIterator.h"
#include "NumericalRange.h"


/* Constructor */
PixelIterator::PixelIterator():
  _isNormalised(false),
  _normalisation(0),
 _hasValidNormalisation(false)
{
  PROC;
  ENDPROC;
};

/* Destructor */
PixelIterator::~PixelIterator()
  {
    PROC;
    ENDPROC;
  };

/* Get the normalised pixel value */
PixelElement PixelIterator::normalisedItem()
{
  PROC;
  
  /* Need to run normalise() in advance of an iteration.
     ( Can't just run normalise() automatically as this
     modifies the current position, which may not be what
     user wanted ) */
  if (!_isNormalised)
    {
      errstr 
	<< "Need to run normalise() in PixelIterator in advance of using normalisedValue()"
	<< Sas::error("NotNormalised");
    }

  if (!_hasValidNormalisation)
    {
      errstr 
	<< "Cannot access a normalised pixel item, as sum of data referenced by PixelIterator is zero" 
	<< Sas::error("invalidNormalisation");
    }

  PixelElement px = currentItem();
  
  RETURN PixelElement( px.xIndex(),
		       px.yIndex(),
		       px.xCentre(),
		       px.yCentre(),
		       px.xWidth(),
		       px.yWidth(),
		       px.value()/_normalisation );
}

bool PixelIterator::hasValidNormalisation()
{
  PROC;
  if (!_isNormalised)
    {
      errstr 
	<< "Need to run normalise() in PixelIterator in advance of using hasValidNormalisation()"
	<< Sas::error("NotNormalised");
    }

  RETURN _hasValidNormalisation;
}
  
void PixelIterator::normalise()
{
  PROC;
  
  _normalisation=0;

  FOREACH(this)
    {
      _normalisation += currentItem().value();
    }
  
  errstr 
    << " The normalisation is " << _normalisation
    << Sas::message(Msg::App, Msg::Verbose);

  _hasValidNormalisation=
    (_normalisation <= FLT64_MIN) ? false : true;

  /* Reset Iterator */
  first();

  _isNormalised = true;
  
  RETURN;
}

/* [HS] START Using new WCSConvertor interface */
CoordPair PixelIterator::physXYmin() const
{
  PROC;

    CoordPair final;
    CoordPair temp = wcsconvertor()->physicalCoords(1,1);
    final.x(temp.x()-wcsconvertor()->xWidth()/2.0);
    final.y(temp.y()-wcsconvertor()->yWidth()/2.0);

  RETURN final;
}

CoordPair PixelIterator::physXYmax() const
{
  PROC;

    unsigned ybins = size()/xbins();
    CoordPair final;
    CoordPair temp = wcsconvertor()->physicalCoords(xbins(), ybins);
    final.x(temp.x()+wcsconvertor()->xWidth()/2.0);
    final.y(temp.y()+wcsconvertor()->yWidth()/2.0);

  RETURN final;
}
/* [HS] END */

