// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: SimpleIterator
// Author: Hassan Siddiqui
// 
// Modifications: See ChangeLog
// 
// $Id: SimpleIterator.h,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

#ifndef SimpleIterator_H
#define SimpleIterator_H

// General classes

#include <iostream>

// SAS-specific classes


// General utility functions
#include "Tracer.h"

// Local functions
#include "GeneralIterator.h"

// ============================================================================

// class SimpleIterator

// An iterator for basic types; simply iterates over a range of numbers.

// ============================================================================

template <class Item>
class SimpleIterator : public GeneralIterator<Item>
{
public:

  /* Constructor - arguments are range over which iterator traverses:
   * firstItem to lastItem INCLUSIVE.
   */
  SimpleIterator(const Item& firstItem,
		 const Item& lastItem) :
    _firstItem (firstItem),
    _lastItem  (lastItem)
    {
      PROC;
      ENDPROC;
    }

  void first()
    {
      PROC;
      _current = _firstItem; 
      ENDPROC;
    }

  void next ()
    {
      PROC;
      //      if (!_current%100) cout << " _current " << _current << endl;
      assert (!isDone());
      _current++;
    };

  
  bool isDone()
    {
      PROC;
      /* Removed = sign, this ensures that when used in for loop (see macro)
       * that iterations goes as 
       * for (x=first; x <= last; x++)
       * thus iterating from first to last INCLUSIVE. */
      RETURN _current > _lastItem;

      /* Iterate from first to last, with last EXCLUDED */
      //      RETURN _current >= _lastItem;

    };

  Item currentItem()
    {
      PROC;
      RETURN _current;
    };

private:
  Item _current;

  Item _firstItem;
  Item _lastItem;
};
#endif
