// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: TablePixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Class representation of an iterator that handles DAL-based maps
// in table format.
//
// Modifications: See ChangeLog
// 
// $Id: TablePixelIterator.h,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

#ifndef TABLEPIXELITERATOR_H
#define TABLEPIXELITERATOR_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "PixelIterator.h"
#include "WCSConvertor.h"
#include "PixelElement.h"

class TablePixelIterator : public PixelIterator
{
public:

  /* Constructors */
  TablePixelIterator( const string & tablename,
		      const string & xcolname,
		      const string & ycolname,
		      const string & valuecolname); 
  
  /* Destructors */
  ~TablePixelIterator();

  /* Iterator methods */
  void first ();
  void next  ();
  bool isDone();
  PixelElement currentItem();

  /* WCSConvertor object */
  const WCSConvertor * wcsconvertor() const;
  
  unsigned  xbins() const;

  /* The total number of elements in map */
  unsigned size()   const;

  /* Get the value of a given array pixel without setting the current item */
  double valueAt(unsigned long index);
  double valueAt(); // value at the current pixel

protected:

  /* Disallow default constructor */
  TablePixelIterator();

  bool _firstcalled;

  unsigned long _index;

  unsigned long _xindexsize;
  unsigned long _yindexsize;

  unsigned long _tableelements;

  DataSet    * _set;

  Table      * _table;

  Column     * _xColumn;
  Column     * _yColumn;
  Column     * _valueColumn;
  
  ColumnData * _xColData;
  ColumnData * _yColData;
  ColumnData * _valueColData;
  
  int16      * _xData;
  int16      * _yData;
  real64     * _valueData;

  WCSConvertor * _wcsconv;

};

#endif
