// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: array2table
// Author: Hassan Siddiqui
//
// Description: converts a DAL array to a DAL table for testing purposes.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: array2table.cc,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"
#include "ArrayPixelIterator.h"

// General utility functions
#include "Tracer.h"
 
// ---------------------------------------------------

// Main function

// ---------------------------------------------------

int main()
{
  PROC; 

  string arraymapname    = "testarraymap.ds:"; 
  string rawtablename    = "rawpixellist.ds:EVENTS";
  string valuecolumnname = "VALUE";

  ArrayPixelIterator * it = 
    new ArrayPixelIterator(arraymapname);

  /* Create a table from the arraymap, containing additional columns 
   * containing the DETX DETY coords (which can be used by attcalc)
   * and the sky coordinates (generated by attcalc) for each pixel */

  DataSet * set = dataSetServer->open(rawtablename, DataSet::Create);

  Table * table = set->addTable(rawtablename, it->size());
  
  Column * valueCol  = table->addColumn(valuecolumnname, 
					Column::Real64, 
					"Value", 
					"");

  Column * detxCol    = table->addColumn("DETX", 
					 Column::Int16, 
					 "x det coords", 
					 "0.05 arcsec");


  Column * detyCol    = table->addColumn("DETY", 
					 Column::Int16, 
					 "y det coords", 
					 "0.05 arcsec");

  Column * indexxCol    = table->addColumn("XPIXEL", 
					 Column::Int16, 
					 "x index", 
					 "");


  Column * indexyCol    = table->addColumn("YPIXEL", 
					 Column::Int16, 
					 "y index", 
					 "");

  ColumnData * valueColData  = valueCol->data();
  real64     * valueData     = valueColData->real64Data();

  ColumnData * detxColData   = detxCol->data();
  int16      * detxData      = detxColData->int16Data();

  ColumnData * detyColData   = detyCol->data();
  int16      * detyData      = detyColData->int16Data();

  ColumnData * indexxColData   = indexxCol->data();
  int16      * indexxData      = indexxColData->int16Data();

  ColumnData * indexyColData   = indexyCol->data();
  int16      * indexyData      = indexyColData->int16Data();

  /* Now iterate through arraymap, 
     adding an entry for each and every pixel */
  int row = 0;

  FOREACH(it)
    {
      PixelElement pixel = it->currentItem();

      errstr 
	<< "TEST: pixel value=" << pixel.value()
	<< " indexx="           << pixel.xIndex()
	<< " indexy="           << pixel.yIndex()
	<< " detx="             << pixel.xCentre()
	<< " dety="             << pixel.yCentre()
 	<< Sas::message(Msg::App, Msg::Verbose);

      valueData[row]  = pixel.value();
      detxData[row]   = (int) pixel.xCentre();
      detyData[row]   = (int) pixel.yCentre();
      indexxData[row] = pixel.xIndex();
      indexyData[row] = pixel.yIndex();
      ++row;
    }

  /* Close dataset  */
  delete indexyColData;
  delete indexxColData;
  delete detyColData;
  delete detxColData;
  delete valueColData;

  dataSetServer->close(set);
 
  RETURN 0;
} 
