// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: testCopyColumn
// Author: Hassan Siddiqui
//
// Description: Test harness for CopyUtils class
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: testCopyColumn.cc,v 1.1 2012/01/25 15:46:31 rsaxton Exp $




// General classes

#include <iostream>


// SAS-specific classes

#include "param.h"
#include "Dal.h"

// General utility functions
#include "CopyUtils.h"
#include "Tracer.h"

static const unsigned long NUMBER_OF_ROWS = 10;
 
// ---------------------------------------------------

// Main function

// ---------------------------------------------------

int main()
{
  PROC;

  // Write dataset

  DataSet* set = dataSetServer->open("testCopyColumn.fits", DataSet::Create);

  Table* fromTable = set->addTable("FROM_TABLE", NUMBER_OF_ROWS);


  // Add a few columns

  // Int16 column

  string namei16 = "TESTI16_COLUMN";

  Column* testi16Col = fromTable->addColumn(namei16, Column::Int16, "int16 test data", "");
  ColumnData* testi16ColData = testi16Col->data();
  int16* testi16Data = testi16ColData->int16Data();
  

  // Real32 column

  string namer32 = "TESTR32_COLUMN";

  Column* testr32Col = fromTable->addColumn(namer32, Column::Real32, "real32 test data", "");
  ColumnData* testr32ColData = testr32Col->data();
  real32* testr32Data = testr32ColData->real32Data();
  
  // Populate the columns

  for (unsigned int i  = 0; i < fromTable->rows(); ++i)
    {
      testi16Data[i] = i*10;
      testr32Data[i] = i*100;

    }

  // Now create a new table

  Table* toTable = set->addTable("TO_TABLE", NUMBER_OF_ROWS);

  // Copy columns from the old to the new table

  const Column* to_i16Col = CopyUtils::copyColumnToTable(testi16Col, toTable);
  
  const Column* to_r32Col = CopyUtils::copyColumnToTable(testr32Col, toTable);
  

  // Copy column from the old to the new table, with a new name

  const Column* copy_to_i16Col = CopyUtils::copyColumnToTable(testi16Col, toTable, "A_COPY");

  // Do some checks

  assert(testi16Col->elements() == to_i16Col->elements());
  assert(testr32Col->elements() == to_r32Col->elements());
  assert(testi16Col->elements() == copy_to_i16Col->elements());

  dataSetServer->close(set);


  RETURN 0;
} 
